@@ -312,8 +312,6 @@ int master_loop(char **argv, char **environ) {
 	int ready_to_reload = 0;
 	int ready_to_die = 0;
 
-	int master_has_children = 0;
-
 	uint8_t uwsgi_signal;
 
 	time_t last_request_timecheck = 0, now = 0;
@@ -662,7 +660,7 @@ int master_loop(char **argv, char **environ) {
 
 		if (uwsgi.to_outworld) {
 			//uwsgi_log("%d/%d\n", uwsgi.lazy_respawned, uwsgi.numproc);
-			if (uwsgi.lazy_respawned >= uwsgi.numproc) {
+			if (uwsgi.lazy_respawned >= uwsgi.numproc || (uwsgi.cheaper && uwsgi.lazy_respawned == uwsgi.cheaper_count)) {
 				uwsgi.to_outworld = 0;
 				uwsgi.master_mercy = 0;
 				uwsgi.lazy_respawned = 0;
@@ -755,40 +753,31 @@ int master_loop(char **argv, char **environ) {
 			return -1;
 		}
 
-		if (!uwsgi.cheap) {
-
-			if (uwsgi.numproc > 0 || ushared->gateways_cnt > 0 || uwsgi.daemons_cnt > 0) {
-				master_has_children = 1;
-			}
-#ifdef UWSGI_SPOOLER
-			if (uwsgi.spoolers) {
-				master_has_children = 1;
-			}
-#endif
-		}
-
-		if (!master_has_children) {
-			diedpid = 0;
-		}
-		else {
-			diedpid = waitpid(WAIT_ANY, &waitpid_status, WNOHANG);
-			if (diedpid == -1) {
-				if (errno == ECHILD && uwsgi.cheaper) {
-					if (uwsgi.to_heaven) {
-						ready_to_reload = uwsgi.numproc;
-						continue;
-					}
-					else if (uwsgi.to_hell) {
-						ready_to_die = uwsgi.numproc;
-						continue;
-					}
-				}
-				uwsgi_error("waitpid()");
-				/* here is better to reload all the uWSGI stack */
-				uwsgi_log("something horrible happened...\n");
-				reap_them_all(0);
-				exit(1);
-			}
+		diedpid = waitpid(WAIT_ANY, &waitpid_status, WNOHANG);
+		if (diedpid == -1) {
+			if (errno == ECHILD) {
+	                        if (uwsgi.to_heaven) {
+	                               	ready_to_reload = uwsgi.numproc;
+	                                continue;
+	                        }
+	                        else if (uwsgi.to_hell) {
+	                        	ready_to_die = uwsgi.numproc;
+	                                continue;
+	                        }
+	                        else if (uwsgi.to_outworld) {
+	                        	uwsgi.lazy_respawned = uwsgi.numproc;
+					uwsgi_log("*** no workers to reload found ***\n");
+	                                continue;
+	                        }
+	                        diedpid = 0;
+	                }
+                        else {
+	                        uwsgi_error("waitpid()");
+	                        /* here is better to reload all the uWSGI stack */
+	                        uwsgi_log("something horrible happened...\n");
+	                        reap_them_all(0);
+	                	exit(1);
+	                }
 		}
 
 		if (diedpid == 0) {
@@ -1211,7 +1200,6 @@ int master_loop(char **argv, char **environ) {
 				else if (last_request_timecheck < uwsgi.current_time && (uwsgi.current_time - last_request_timecheck > uwsgi.idle)) {
 					uwsgi_log("workers have been inactive for more than %d seconds (%llu-%llu)\n", uwsgi.idle, (unsigned long long) uwsgi.current_time, (unsigned long long) last_request_timecheck);
 					uwsgi.cheap = 1;
-					master_has_children = 0;
 					if (uwsgi.die_on_idle) {
 						if (uwsgi.has_emperor) {
 							char byte = 22;
@@ -1231,7 +1219,8 @@ int master_loop(char **argv, char **environ) {
 							continue;
 						kill(uwsgi.workers[i].pid, SIGKILL);
 						if (waitpid(uwsgi.workers[i].pid, &waitpid_status, 0) < 0) {
-							uwsgi_error("waitpid()");
+							if (errno != ECHILD)
+								uwsgi_error("waitpid()");
 						}
 					}
 					uwsgi_add_sockets_to_queue(uwsgi.master_queue);
@@ -1420,7 +1409,11 @@ int master_loop(char **argv, char **environ) {
 
 		}
 
-		if (diedpid > 0) {
+
+		// no one died
+		if (diedpid <= 0)
+			continue;
+
 			// check for deadlocks first
 			struct uwsgi_lock_item *uli = uwsgi.registered_locks;
 			while(uli) {
@@ -1444,7 +1437,7 @@ int master_loop(char **argv, char **environ) {
 nextlock:
 				uli = uli->next;
 			}
-		}
+
 		// reload gateways and daemons only on normal workflow
 		if (!uwsgi.to_heaven && !uwsgi.to_hell) {
 
@@ -1470,7 +1463,7 @@ nextlock:
 			pid_found = 0;
 			for (i = 0; i < uwsgi.mules_cnt; i++) {
 				if (uwsgi.mules[i].pid == diedpid) {
-					uwsgi_log("OOOPS mule %d crippled...trying respawn...\n", i + 1);
+					uwsgi_log("OOOPS mule %d (pid: %d) crippled...trying respawn...\n", i + 1, uwsgi.mules[i].pid);
 					uwsgi_mule(i + 1);
 					pid_found = 1;
 					break;
