@@ -492,6 +492,12 @@ int http_parse(struct http_session *h_session) {
 		hv = hv->next;
 	}
 
+	// security check
+	if (c >= MAX_HTTP_VEC-4) {
+		uwsgi_log("too much headers in request. skipping it.\n");
+		return 0;
+	}
+
 #ifdef UWSGI_DEBUG
 	uwsgi_log("vec size: %d pkt size: %d load %d\n", c, h_session->uh.pktsize, uhttp.load);
 #endif
@@ -508,7 +514,7 @@ void http_loop(int id) {
 	ssize_t len;
 	int i, j;
 
-	char *magic_table[0xff];
+	char *magic_table[256];
 
 	char bbuf[UMAX16];
 
