@@ -134,21 +134,21 @@ XS(XS_input_read) {
                 sv_setpvn(read_buf, wsgi_req->post_buffering_buf, remains);
                 bytes = remains;
                 wsgi_req->post_pos += remains;
-
+		goto ret;
         }
 
         tmp_buf = uwsgi_malloc(remains);
 
         if (uwsgi_waitfd(fd, uwsgi.shared->options[UWSGI_OPTION_SOCKET_TIMEOUT]) <= 0) {
                 free(tmp_buf);
-                croak("error waiting for wsgi.input data");
+                croak("error waiting for psgi.input data");
                 goto ret;
         }
 
         bytes = read(fd, tmp_buf, remains);
         if (bytes < 0) {
                 free(tmp_buf);
-                croak("error reading wsgi.input data");
+                croak("error reading psgi.input data");
                 goto ret;
         }
 
