@@ -709,7 +709,8 @@ PyObject *py_uwsgi_send(PyObject * self, PyObject * args) {
 	PyObject *data;
 	PyObject *arg1, *arg2;
 
-	int uwsgi_fd = uwsgi.wsgi_req->poll.fd;
+	struct wsgi_request *wsgi_req = current_wsgi_req();
+	int uwsgi_fd = wsgi_req->poll.fd;
 
 	if (!PyArg_ParseTuple(args, "O|O:send", &arg1, &arg2)) {
 		return NULL;
@@ -1595,6 +1596,9 @@ PyObject *py_uwsgi_send_spool(PyObject * self, PyObject * args, PyObject *kw) {
 	char *body = NULL;
 	size_t body_len= 0;
 
+	// this is a counter for non-request-related tasks (like threads or greenlet)
+	static int internal_counter = 0xffff;
+
 	struct uwsgi_spooler *uspool = uwsgi.spoolers;
 
 	spool_dict = PyTuple_GetItem(args, 0);
@@ -1726,20 +1730,36 @@ PyObject *py_uwsgi_send_spool(PyObject * self, PyObject * args, PyObject *kw) {
 		}
 	}
 
-	UWSGI_RELEASE_GIL
-
 	if (numprio) {
 		priority = uwsgi_num2str(numprio);
 	} 
-	i = spool_request(uspool, spool_filename, uwsgi.workers[0].requests + 1, wsgi_req->async_id, spool_buffer, cur_buf - spool_buffer, priority, at, body, body_len);
+
+
+	int async_id; 
+ 		 
+ 	if (wsgi_req) { 
+ 		async_id = wsgi_req->async_id; 
+ 	} 
+       	else { 
+ 	                // the GIL is protecting this counter... 
+ 	        async_id = internal_counter; 
+ 		internal_counter++; 
+ 	} 
+ 	
+ 	UWSGI_RELEASE_GIL 
+ 		 
+ 	i = spool_request(uspool, spool_filename, uwsgi.workers[0].requests + 1, async_id, spool_buffer, cur_buf - spool_buffer, priority, at, body, body_len); 
+ 	 
+ 	UWSGI_GET_GIL 
+
+
+
 	if (priority) {
 		free(priority);
 	}
 		
 	free(spool_buffer);
 
-	UWSGI_GET_GIL
-
 	Py_DECREF(spool_vars);
 
 	if (i > 0) {
@@ -2490,6 +2510,11 @@ PyObject *py_uwsgi_workers(PyObject * self, PyObject * args) {
 			goto clear;
 		}
 
+		apps_tuple = PyDict_GetItemString(worker_dict, "apps");
+		if (apps_tuple) {
+			Py_DECREF(apps_tuple);
+		}	
+
 		PyDict_Clear(worker_dict);
 
 		zero = PyInt_FromLong(uwsgi.workers[i + 1].id);
@@ -2599,21 +2624,37 @@ PyObject *py_uwsgi_workers(PyObject * self, PyObject * args) {
 			apps_dict = PyDict_New();
 			ua = &uwsgi.workers[i+1].apps[j];
 
-			PyDict_SetItemString(apps_dict, "id", PyInt_FromLong(j));
+			zero = PyInt_FromLong(j);
+			PyDict_SetItemString(apps_dict, "id", zero);
+			Py_DECREF(zero);
 
-			PyDict_SetItemString(apps_dict, "modifier1", PyInt_FromLong(ua->modifier1));
+			zero = PyInt_FromLong(ua->modifier1);
+			PyDict_SetItemString(apps_dict, "modifier1", zero);
+			Py_DECREF(zero);
 
 			zero = PyString_FromStringAndSize(ua->mountpoint, ua->mountpoint_len);
 			PyDict_SetItemString(apps_dict, "mountpoint", zero);
 			Py_DECREF(zero);
 
-			PyDict_SetItemString(apps_dict, "startup_time", PyInt_FromLong((long) ua->startup_time));
+			zero = PyInt_FromLong((long) ua->startup_time);
+			PyDict_SetItemString(apps_dict, "startup_time", zero);
+			Py_DECREF(zero);
 
-			PyDict_SetItemString(apps_dict, "interpreter", PyInt_FromLong((long)ua->interpreter));
-			PyDict_SetItemString(apps_dict, "callable", PyInt_FromLong((long)ua->interpreter));
+			zero = PyInt_FromLong((long)ua->interpreter);
+			PyDict_SetItemString(apps_dict, "interpreter", zero);
+			Py_DECREF(zero);
 
-			PyDict_SetItemString(apps_dict, "requests", PyLong_FromUnsignedLongLong(ua->requests));
-			PyDict_SetItemString(apps_dict, "exceptions", PyLong_FromUnsignedLongLong(ua->exceptions));
+			zero = PyInt_FromLong((long)ua->callable);
+			PyDict_SetItemString(apps_dict, "callable", zero);
+			Py_DECREF(zero);
+
+			zero = PyLong_FromUnsignedLongLong(ua->requests);
+			PyDict_SetItemString(apps_dict, "requests", zero);
+			Py_DECREF(zero);
+
+			zero = PyLong_FromUnsignedLongLong(ua->exceptions);
+			PyDict_SetItemString(apps_dict, "exceptions", zero);
+			Py_DECREF(zero);
 
 			if (ua->chdir) {
 				zero = PyString_FromString(ua->chdir);
@@ -3648,12 +3689,29 @@ PyObject *py_uwsgi_cache_get(PyObject * self, PyObject * args) {
 
 }
 
+PyObject *py_uwsgi_cache_clear(PyObject * self, PyObject * args) { 
+	uint64_t i; 
+ 	// skip the first slot 
+ 	for (i = 1; i < uwsgi.cache_max_items; i++) { 
+ 		UWSGI_RELEASE_GIL 
+ 	        uwsgi_wlock(uwsgi.cache_lock); 
+ 		uwsgi_cache_del(NULL, 0, i); 
+ 	        uwsgi_rwunlock(uwsgi.cache_lock); 
+ 		UWSGI_GET_GIL 
+ 	} 
+ 	 
+	Py_INCREF(Py_None); 
+	return Py_None; 
+	
+}
+
 static PyMethodDef uwsgi_cache_methods[] = {
 	{"cache_get", py_uwsgi_cache_get, METH_VARARGS, ""},
 	{"cache_set", py_uwsgi_cache_set, METH_VARARGS, ""},
 	{"cache_update", py_uwsgi_cache_update, METH_VARARGS, ""},
 	{"cache_del", py_uwsgi_cache_del, METH_VARARGS, ""},
 	{"cache_exists", py_uwsgi_cache_exists, METH_VARARGS, ""},
+	{"cache_clear", py_uwsgi_cache_clear, METH_VARARGS, ""},
 	{NULL, NULL},
 };
 
