@@ -341,6 +341,7 @@ int uwsgi_request_wsgi(struct wsgi_request *wsgi_req) {
 
 #ifdef UWSGI_ASYNC
 	if (wsgi_req->async_status == UWSGI_AGAIN) {
+		UWSGI_GET_GIL
 		// get rid of timeout
 		if (wsgi_req->async_timed_out) {
 			PyDict_SetItemString(wsgi_req->async_environ, "x-wsgiorg.fdevent.timeout", Py_True);
@@ -357,7 +358,9 @@ int uwsgi_request_wsgi(struct wsgi_request *wsgi_req) {
 		else {
 			PyDict_SetItemString(wsgi_req->async_environ, "uwsgi.ready_fd", Py_None);
 		}
-		return manage_python_response(wsgi_req);
+		int ret = manage_python_response(wsgi_req);
+		UWSGI_RELEASE_GIL
+		return ret;
 	}
 #endif
 
@@ -439,20 +442,13 @@ int uwsgi_request_wsgi(struct wsgi_request *wsgi_req) {
 	}
 
 
-#ifdef UWSGI_ASYNC
-	wsgi_req->async_environ = wi->environ[wsgi_req->async_id];
-	wsgi_req->async_args = wi->args[wsgi_req->async_id];
-#else
-	wsgi_req->async_environ = wi->environ;
-	wsgi_req->async_args = wi->args;
-#endif
-
 	UWSGI_GET_GIL
 
 	// no fear of race conditions for this counter as it is already protected by the GIL
 	wi->requests++;
 
-	Py_INCREF((PyObject *)wsgi_req->async_environ);
+	// create WSGI environ
+	wsgi_req->async_environ = up.wsgi_env_create(wsgi_req, wi);
 
 	wsgi_req->async_result = wi->request_subhandler(wsgi_req, wi);
 
@@ -462,6 +458,7 @@ int uwsgi_request_wsgi(struct wsgi_request *wsgi_req) {
 		while (wi->response_subhandler(wsgi_req) != UWSGI_OK) {
 #ifdef UWSGI_ASYNC
 			if (uwsgi.async > 1) {
+				UWSGI_RELEASE_GIL
 				return UWSGI_AGAIN;
 			}
 			else {
@@ -515,7 +512,7 @@ int uwsgi_request_wsgi(struct wsgi_request *wsgi_req) {
                 Py_DECREF((PyObject *)wsgi_req->async_input);
         }
         if (wsgi_req->async_environ) {
-                PyDict_Clear(wsgi_req->async_environ);
+		up.wsgi_env_destroy(wsgi_req);
         }
 
 	UWSGI_RELEASE_GIL
