@@ -3,6 +3,7 @@
 extern struct uwsgi_server uwsgi;
 
 struct uwsgi_rack ur;
+struct uwsgi_plugin rack_plugin;
 
 struct uwsgi_option uwsgi_rack_options[] = {
 
@@ -272,6 +273,7 @@ void uwsgi_ruby_gem_set_apply(char *gemset) {
 
 	char *ptr = buffer;
 
+
 	for(i=0;i<size;i++) {
 		if (buffer[i] == '\n') {
 			buffer[i] = 0;
@@ -282,7 +284,9 @@ void uwsgi_ruby_gem_set_apply(char *gemset) {
 		}
 	}
 
-	free(buffer);
+	// do not free the buffer
+        // environ will reuse it !!!
+	//free(buffer);
 
 	if (waitpid(pid, &waitpid_status, 0) <0) {
 		uwsgi_error("waitpid()");
@@ -322,6 +326,9 @@ void uwsgi_ruby_gemset(char *gemset) {
                 free(filename);
 		rvm_paths = rvm_paths->next;
 	}
+
+	uwsgi_log("ERROR: unable to load gemset %s !!!\n", gemset);
+	exit(1);
 	
 }
 
@@ -505,7 +512,7 @@ ready:
 	rb_define_method(ur.rb_uwsgi_io_class, "read", rb_uwsgi_io_read, -2);
 	rb_define_method(ur.rb_uwsgi_io_class, "rewind", rb_uwsgi_io_rewind, 0);
 
-	struct uwsgi_app *ua = uwsgi_add_app(ur.app_id, 7, (char*)"", 0, NULL, NULL);
+	struct uwsgi_app *ua = uwsgi_add_app(ur.app_id, rack_plugin.modifier1, (char*)"", 0, NULL, NULL);
 	ua->started_at = now;
 	ua->startup_time = uwsgi_now() - now;
 
@@ -726,6 +733,10 @@ int uwsgi_rack_request(struct wsgi_request *wsgi_req) {
 
 	rb_hash_aset(env, rb_str_new2("rack.errors"), rb_funcall( rb_const_get(rb_cObject, rb_intern("IO")), rb_intern("new"), 2, INT2NUM(2), rb_str_new("w",1) ));
 
+	// remove HTTP_CONTENT_LENGTH and HTTP_CONTENT_TYPE
+        rb_hash_delete(env, rb_str_new2("HTTP_CONTENT_LENGTH"));
+        rb_hash_delete(env, rb_str_new2("HTTP_CONTENT_TYPE"));
+
 	ret = rb_protect( call_dispatch, env, &error);
 	if (error) {
 		uwsgi_ruby_exception();
